/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Oct 21, 2008
 */

package eu.moonlight3d.common.document.extensible;

import java.util.ArrayList;

import eu.moonlight3d.framework.document.Document;
import eu.moonlight3d.framework.document.DocumentFactory;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.helper.PropertyContainer;

public class ExtensibleDocumentFactory implements DocumentFactory {
	
	private ArrayList<DocumentContributionFactory> contributionFactories;
	private FileType fileType;
	private String typeName;
	private String displayName;
	
	public ExtensibleDocumentFactory(String typeName, String displayName, FileType fileType) {
		this.typeName=typeName;
		this.displayName=displayName;
		this.fileType=fileType;
		contributionFactories=new ArrayList<DocumentContributionFactory>();
	}

	public boolean canOpenDocument(String filename) {
		// TODO Auto-generated method stub
		return true;
	}

	public Document createNewDocument() {
		return createNewDocument(createNewDocumentProperties());
	}

	public Document createNewDocument(PropertyContainer options) {
		ExtensibleDocument document=new ExtensibleDocument(typeName,this);
		
		document.initialize(options);
		
		return document;
	}

	public PropertyContainer createNewDocumentProperties() {
		PropertyContainer container=new PropertyContainer();
		
		for(DocumentContributionFactory factory : contributionFactories) {
			factory.registerNewContributionProperties(container);
		}
		
		return container;
	}

	public String getDisplayName() {
		return displayName;
	}

	public FileType getFileType() {
		return fileType;
	}

	public String getTypeName() {
		return typeName;
	}

	public boolean canCreateNew() {
		return true;
	}

	public boolean canSaveFiles() {
		return true;
	}

	public Document openDocument(String filename) throws Exception {
		return new ExtensibleDocument(typeName,this,filename);
	}

	
	public void registerDocumentContributionFactory(DocumentContributionFactory contributionFactory) {
		contributionFactories.add(contributionFactory);
	}
	
	public void unregisterDocumentContributionFactory(DocumentContributionFactory contributionFactory) {
		contributionFactories.remove(contributionFactory);
	}
	
	ArrayList<DocumentContributionFactory> getDocumentContributionFactories() {
		return (ArrayList<DocumentContributionFactory>) contributionFactories.clone();
	}

}
