/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Apr 19, 2006
 */
package eu.moonlight3d.common.file.core;

import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.helper.PropertyContainer;

/**
 * A general interface for importing from and exporting to
 * foreign file formats.
 *
 * @author gregor
 */
public abstract class FileFormat {
	/**
	 * Get the file type information for this file format.
	 * 
	 * @return file type information
	 */
	public abstract FileType getFileType();
	
	/**
	 * Check if this FileFormat instance is able to import
	 * files at all.
	 * 
	 * @return true if this FileFormat can import files
	 */
	public abstract boolean canImport();

	/**
	 * Check if this FileFormat instance is able to export
	 * files at all.
	 * 
	 * @return true if this FileFormat can export files
	 */
	public abstract boolean canExport();
	
	/**
	 * Get a property container with all properties that control
	 * the import behaviour or null if there are none.
	 * 
	 * @return import properties
	 */
	public abstract PropertyContainer getImportProperties();
	
	/**
	 * Get a property container with all properties that control
	 * the export behaviour or null if there are none.
	 * 
	 * @return export properties
	 */
	public abstract PropertyContainer getExportProperties();
	
	/**
	 * Import a given file.
	 * 
	 * @param filename the name of the file to import
	 * @throws Exception if the import fails
	 */
	public abstract void importFile(String filename) throws Exception;

	/**
	 * Export the current main document to the given file.
	 * 
	 * @param filename the name of the file to export to
	 * @throws Exception if the export fails
	 */
	public abstract void exportFile(String filename) throws Exception;
}
