/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Apr 19, 2006
 */
package eu.moonlight3d.common.file.core;

import java.util.ArrayList;
import java.util.List;

/**
 * This file format manager keeps track of various export and import
 * plugins. 
 *
 * @author gregor
 */
public class Manager {
	/**
	 * Reference to singleton instance.
	 */
	private static Manager instance;
	
	/**
	 * Return singleton instance of this class.
	 * 
	 * @return Manager instance
	 */
	public static Manager getInstance() {
		if(instance==null) {
			instance=new Manager();
		}
		
		return instance;
	}
	
	/**
	 * List of registered file format.s
	 */
	private ArrayList<FileFormat> fileFormats;
	
	/**
	 * Internal constructor.
	 */
	private Manager() {
		fileFormats=new ArrayList<FileFormat>();
	}
	
	/**
	 * Register a new file format.
	 * 
	 * @param format file format to register
	 */
	public void registerFileFormat(FileFormat format) {
		fileFormats.add(format);
	}
	
	/**
	 * Unregister a file format.
	 * 
	 * @param formatName file format to unregister.
	 */
	public void unregisterFileFormat(String formatName) {
		for(FileFormat format : fileFormats) {
			if(format.getFileType().getName().equals(formatName)) {
				fileFormats.remove(format);
				return;
			}
		}
	}
	
	/**
	 * Get a list of all registered file formats.
	 * 
	 * @return all registered file formats
	 */
	public List<FileFormat> getFileFormats() {
		return (ArrayList<FileFormat>) fileFormats.clone();
	}
}
