/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Apr 19, 2006
 */
package eu.moonlight3d.common.file.importfile;

import java.util.HashMap;
import java.util.List;


import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.file.core.FileFormat;
import eu.moonlight3d.common.file.core.Manager;
import eu.moonlight3d.common.file.core.SettingsDialog;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;

/**
 * Action that provides the "Import..." menu entry.
 *
 * @author gregor
 */
public class ImportAction extends Action {
	/**
	 * Default action constructor.
	 * 
	 * @param parent parent widget, if any
	 */
	public ImportAction(QWidget parent) {
		super("ImportAction",L10N.translate("Import file"),parent);
		triggered.connect(this,"run()");
	}
	
	@Override
	public void run() throws Exception {
		State state=State.getInstance();
		Manager formatManager=Manager.getInstance();
		List<FileFormat> fileFormats=formatManager.getFileFormats();
		
		for(int i=0;i<fileFormats.size();i++) {
			if(!fileFormats.get(i).canImport()) {
				fileFormats.remove(i);
				// decrement here to counter increment in for loop
				i--; 
			}
		}
		
		if(fileFormats.size()==0) {
			String boxTitle=L10N.translate("No file import handlers present");
			String boxMessage=L10N.translate("There are no file import handlers loaded. No file can be imported.");
			QMessageBox.warning(state.getMainWindow(),boxTitle,boxMessage);
			return;
		}
		
		// Prepare file dialog
		HashMap<String, FileFormat> fileFormatMap=new HashMap<String, FileFormat>();
        String filterNames="";
        for(FileFormat fileFormat : fileFormats) {
        	FileType fileType=fileFormat.getFileType();
        	String filterName=fileType.getName() + " (";
        	for(String pattern : fileType.getPatterns()) {
        		filterName+=pattern + " ";
        	}
        	filterName+=")";
         	fileFormatMap.put(filterName, fileFormat);
        	filterNames=filterNames+filterName+ ";;";
        }
        filterNames+=L10N.translate("By filename");
        
        QFileDialog.Filter filter=new QFileDialog.Filter(filterNames);
        
        // Let user select file
        String filename = QFileDialog.getOpenFileName(state.getMainWindow(),L10N.translate("Import file"),"",filter);        
        if(filename.equals("")) {
        	return;
        }
        
        // Find matching importer
        FileFormat importer=null;
        if(filter.selectedFilter.equals(L10N.translate("By filename"))) {
        	for(FileFormat fileFormat : fileFormats) {
        		if(fileFormat.getFileType().matches(filename)) {
        			importer=fileFormat;
        			break;
        		}
        	} 
        } else {
        	importer=fileFormatMap.get(filter.selectedFilter);
        }
        if(importer==null) {
        	String boxTitle=L10N.translate("No matching import handler");
        	String boxMessage=L10N.translate("There seems to be no suitable import handler for the given file. The file cannot be imported.");
			QMessageBox.critical(state.getMainWindow(),boxTitle,boxMessage);
			return;        	
        }
        
        PropertyContainer properties=importer.getImportProperties();
        if(properties!=null && properties.getNumProperties()!=0) {
        	SettingsDialog dialog=new SettingsDialog(state.getMainWindow(),properties,L10N.translate("Importer settings"));
        	int result=dialog.exec();
        	if(result==QDialog.DialogCode.Rejected.value()) {
        		return;
        	}
        }
        
        importer.importFile(filename);
	}
}
