/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Nov 9, 2008
 */

package eu.moonlight3d.common.ui.command;

/**
 * CommandStack listeners. Registered listeners to the command stack
 * get notified of changes in the stack.
 *
 * @author gregor
 */
public interface CommandStackListener {
	/**
	 * This function is called when the maximum size of the command stack
	 * has been changed.
	 */
	public void stackSizeChanged();
	
	/**
	 * This function is called when a new command is about to be executed
	 * for the first time.
	 * 
	 * @param command the command to be executed
	 */
	public void beforeExecute(Command command);
	
	/**
	 * This function is called when a new command finished execution for the
	 * first time.
	 * 
	 * @param command the command that was executed
	 * @param successful true if execution was successful, false otherwise
	 */
	public void afterExecute(Command command, boolean successful);
	
	/**
	 * This function is called when a command is about to be undone
	 * 
	 * @param command the command to be undone
	 */
	public void beforeUndo(Command command);

	/**
	 * This function is called when a command that was undone
	 * 
	 * @param command the command that was undone
	 */
	public void afterUndo(Command command);

	/**
	 * This function is called when a command is about to be redone
	 * 
	 * @param command the command to be redone
	 */
	public void beforeRedo(Command command);

	/**
	 * This function is called when a command that was redone
	 * 
	 * @param command the command that was redone
	 */
	public void afterRedo(Command command);
}
