/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Nov 9, 2008
 */

package eu.moonlight3d.common.ui.command;

import com.trolltech.qt.core.QObject;

import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;

public class RedoAction extends Action {
	private class RedoCommandStackListener implements CommandStackListener {

		public void afterExecute(Command command, boolean successful) {
			updateAction();
		}

		public void afterRedo(Command command) {
			updateAction();
		}

		public void afterUndo(Command command) {
			updateAction();
		}

		public void beforeExecute(Command command) {
			// nothing to do here
		}

		public void beforeRedo(Command command) {
			// nothing to do here
		}

		public void beforeUndo(Command command) {
			// nothing to do here
		}

		public void stackSizeChanged() {
			updateAction();
		}
		
	}
	
	private RedoCommandStackListener redoCommandStackListener;
	
	public RedoAction(String name, QObject parent) {
		super(name, parent);
		
		CommandStack commandStack=CommandStack.getInstance();
		redoCommandStackListener=new RedoCommandStackListener();
		commandStack.registerCommandStackListener(redoCommandStackListener);
		triggered.connect(this,"run()");
	}

	private void updateAction() {
		CommandStack commandStack=CommandStack.getInstance();
		if(commandStack.canUndo()) {
			setEnabled(true);
			setText(L10N.translate("Redo: ") + commandStack.getCurrentRedoCommand().getBriefDescription());
		} else {
			setEnabled(false);
			setText(L10N.translate("Redo: not possible"));
		}
	}

	public void run() {
		CommandStack commandStack=CommandStack.getInstance();
		if(commandStack.canRedo()) {
			commandStack.redo();
		}
	}
}
