/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Sep 20, 2007
 */

package eu.moonlight3d.common.ui.coreactions;

import java.text.MessageFormat;
import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;


public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.common.ui.mainwindow.Plugin");
		return dependencyList;
	}

	public String getName() {
		return "eu.moonlight3d.common.ui.coreactions.Plugin";
	}

	public void load() throws Exception {
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager manager=state.getUIManager();
			
			manager.registerAction(new AboutAction(null));
			manager.registerAction(new CloseCurrentFrameAction(null));
			manager.registerAction(new ExitAction(null));
			
	        // add menu contributions for core actions
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"A_File","menubar");
			MenuNode fileMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("File"),"");
			fileMenu.add(new MenuNode(MenuNode.Type.ExtensionPoint,"fileMenuExtensionPoint",""));
			fileMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Exit"),"ExitAction"));
			contribution.add(fileMenu);
			manager.getLayout("mainwindow").addMenuContribution(contribution);
			
			contribution=new MenuNode(MenuNode.Type.Contribution,"Z_Help","menubar");
			MenuNode helpMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("Help"),"");
			helpMenu.add(new MenuNode(MenuNode.Type.ExtensionPoint,"helpMenuExtensionPoint",""));
			String aboutEntryName=MessageFormat.format(L10N.translate("About {0}"),state.getVersionInformation().getApplicationName());
			helpMenu.add(new MenuNode(MenuNode.Type.Entry,aboutEntryName,"AboutAction"));
			contribution.add(helpMenu);
			manager.getLayout("mainwindow").addMenuContribution(contribution);
		}
	}

	public void unload() throws Exception {
		State state=State.getInstance();
		if(!state.getBatchMode()) {
			UIManager manager=state.getUIManager();
			
			manager.unregisterAction("AboutAction");
			manager.unregisterAction("CloseCurrentFrameAction");
			manager.unregisterAction("ExitAction");
		}
	}

}
