/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 6, 2005
 */
package eu.moonlight3d.common.ui.fileactions;


import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QWidget;
import com.trolltech.qt.gui.QMessageBox.StandardButton;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.document.DocumentFactory;
import eu.moonlight3d.framework.document.DocumentManager;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;

/**
 * Implementation of the New action creating a new main document
 * after asking the user if he wishes to discard the changes, should
 * there be any in the document. It creates a new main document
 * using the default document factory name set in the 
 * {@link DocumentManager}.
 *
 * @author gregor
 */
public class NewAction extends Action {
	/**
	 * Constructor taking a QWidget as a parent
	 * 
	 * @param parent parent widget
	 */
	public NewAction(QWidget parent) {
		super("NewAction",L10N.translate("&New..."),parent);
		setShortcut("Ctrl+N");
		setToolTip(L10N.translate("Create a new file"));
		setStatusTip(L10N.translate("Create a new file"));
		triggered.connect(this,"run()");
	}

	@Override
	public void run() {
		DocumentManager documentManager=State.getInstance().getDocumentManager();
		if(documentManager.getMainDocument()!=null) {
			if(documentManager.getMainDocument().hasUnsavedChanges()) {
				String boxTitle=L10N.translate("Unsaved changes");
				String boxMessage=L10N.translate("The current file contains unsaved changes. Do you wish to discard them?");
				StandardButton result=QMessageBox.question(null,boxTitle,boxMessage, new QMessageBox.StandardButtons(StandardButton.Yes, StandardButton.No));
				if(result!=StandardButton.Yes) {
					return;
				}
			}
		}
		
		if(documentManager.getDefaultDocumentFactory()==null) {
			return;
		}
		
		String factoryName=documentManager.getDefaultDocumentFactory();
		DocumentFactory factory=documentManager.getDocumentFactory(factoryName);
		PropertyContainer options=factory.createNewDocumentProperties();
		
		if(options!=null && options.getNumProperties()!=0) {
			// options exist, show a property editor dialog for them
			QWidget currentWindow=State.getInstance().getUIManager().getActiveApplicationWindow();
			NewDocumentOptionsDialog dialog=new NewDocumentOptionsDialog(currentWindow,options);
			dialog.exec();
			
			if(!dialog.getOKPressed()) {
				return;
			}
			
			documentManager.createNewMainDocument(factoryName,options);
		} else {
			documentManager.createNewMainDocument(factoryName);
		}
	}
}
