/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Feb 3, 2008
 */

package eu.moonlight3d.common.ui.fileactions;

import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.ui.propertysheet.PropertySheet;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;

/**
 * Display a dialog with options for new files during
 * the creation of a new file.
 *
 * @author gregor
 */
public class NewDocumentOptionsDialog extends QDialog {
	/**
	 * The dialog's OK button
	 */
	private QPushButton okButton;
	/**
	 * The dialog's cancel button
	 */
	private QPushButton cancelButton;
	/**
	 * The dialog's property sheet
	 */
	private PropertySheet propertySheet;
	/**
	 * Flag used to remember how the dialog was closed.
	 */
	private boolean okPressed=false;
	
	/**
	 * Default constructor taking the parent window and the
	 * options to display as parameters.
	 * 
	 * @param parent the dialog's parent widget
	 * @param options the options to display
	 */
	public NewDocumentOptionsDialog(QWidget parent, PropertyContainer options) {
		super(parent);
		
		setWindowTitle(L10N.translate("Create a new file"));

		// property sheet, OK and cancel buttons
		propertySheet=new PropertySheet(this);
		propertySheet.setPropertyContainer(options);
		
		okButton=new QPushButton(this);
		okButton.setText(L10N.translate("OK"));
		okButton.clicked.connect(this,"onOK()");
		
		cancelButton=new QPushButton(this);
		cancelButton.setText(L10N.translate("Cancel"));
		cancelButton.clicked.connect(this,"onCancel()");
		
		// TODO dialog layout, remember if OK or cancel was pressed
		
		QVBoxLayout layout=new QVBoxLayout();
		QHBoxLayout bottomLayout=new QHBoxLayout();
		bottomLayout.addStretch(1);
		bottomLayout.addWidget(cancelButton);
		bottomLayout.addWidget(okButton);
		
		layout.addWidget(propertySheet);
		layout.addLayout(bottomLayout);
		
		setLayout(layout);
	}

	/**
	 * Handle click on dialog's OK button.
	 */
	protected void onOK() {
		okPressed=true;
		close();
	}
	
	/**
	 * Handle click on dialog's cancel button.
	 */
	protected void onCancel() {
		close();
	}
	
	/**
	 * Return, whether the user clicked OK to close
	 * the dialog.
	 * 
	 * @return true if the dialog was closed with OK, false otherwise
	 */
	public boolean getOKPressed() {
		return okPressed;
	}
}
