/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 3, 2005
 */
package eu.moonlight3d.common.ui.fileactions;


import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QMessageBox;
import com.trolltech.qt.gui.QWidget;
import com.trolltech.qt.gui.QMessageBox.StandardButton;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.document.DocumentFactory;
import eu.moonlight3d.framework.document.DocumentManager;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;
import eu.moonlight3d.framework.ui.ExceptionDialog;

/**
 * Implementation of Open action. This asks the user if he 
 * wishes to discard the changes in the main document if there
 * are any, then displays a file dialog in which the user
 * can select the file to open and loads the selected file as
 * the new main document.
 *
 * @author gregor
 */
public class OpenAction extends Action {
	/**
	 * Constructor taking a QWidget as a parent
	 * 
	 * @param parent parent widget
	 */
	public OpenAction(QWidget parent)
	{
		super("OpenAction",L10N.translate("&Open..."),parent);
		setShortcut("Ctrl+O");
		setToolTip(L10N.translate("Open a file"));
		setStatusTip(L10N.translate("Open a file"));
		triggered.connect(this,"run()");
	}

	@Override
	public void run() {
        State state=State.getInstance();

		DocumentManager documentManager=state.getDocumentManager();
		if(documentManager.getMainDocument()!=null) {
			if(documentManager.getMainDocument().hasUnsavedChanges()) {
				String boxTitle=L10N.translate("Unsaved changes");
				String boxMessage=L10N.translate("The current file contains unsaved changes. Do you wish to discard them?");
				StandardButton result=QMessageBox.question(null,boxTitle,boxMessage, new QMessageBox.StandardButtons(StandardButton.Yes, StandardButton.No));
				if(result!=StandardButton.Yes) {
					return;
				}
			}
		}

        // create filter from registered document factories
		String filterNames="";
		for(DocumentFactory factory : documentManager.getDocumentFactories()) {
			FileType fileType=factory.getFileType();
			filterNames+=fileType.getName() + " ("; 
			for(String pattern : factory.getFileType().getPatterns()) {
				filterNames+=pattern + " ";
			}
			filterNames+=");;";
		}
		filterNames+=L10N.translate("By filename") + " (*)";
		
        QFileDialog.Filter filter=new QFileDialog.Filter(filterNames);

        String filename=QFileDialog.getOpenFileName(state.getMainWindow(),"Open scene","",filter);
        
        if(filename.equals("")) {
            return;
        }

        state.logger().info(L10N.translate("reading from file ") + filename);
        
        State.getInstance().getDocumentManager().closeMainDocument();
        try {
			if(State.getInstance().getDocumentManager().openMainDocument(filename)==null) {
				QMessageBox.critical(state.getMainWindow(),	L10N.translate("Open failed"), L10N.translate("The file could not be opened for an unspecified reason."));
			}
		} catch (Exception e) {
			ExceptionDialog.run(e);
		}
	}
}
