/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 3, 2005
 */
package eu.moonlight3d.common.ui.fileactions;


import java.text.MessageFormat;

import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.document.DocumentFactory;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;
import eu.moonlight3d.framework.ui.ExceptionDialog;

/**
 * Implementation of the Save action found in the file menu.
 * It saves the main document under it's file name if it
 * exists or queries the user for a file dialog if that
 * file name is not set. 
 *
 * @author gregor
 */
public class SaveAction extends Action {
	/**
	 * Constructor taking a QWidget as a parent
	 * 
	 * @param parent parent widget
	 */
	public SaveAction(QWidget parent)
	{
		super("SaveAction",L10N.translate("&Save"),parent);
		setShortcut("Ctrl+S");
		setToolTip(L10N.translate("Save current document to file"));
		setStatusTip(L10N.translate("Save current document to file"));
		triggered.connect(this,"run()");
	}

	@Override
	public void run() {
		State state=State.getInstance();
		
		if(state.getDocumentManager().getMainDocument()==null) {
			return;
		}
		
		if(state.getDocumentManager().getMainDocument().getFilename()!=null) {

	        // save document to the last used filename using the document manager
	        try {
	        	String filename=state.getDocumentManager().getMainDocument().getFilename();
		        state.logger().info(MessageFormat.format(L10N.translate("writing to file {0}"), filename));
				state.getDocumentManager().saveMainDocument(filename);
			} catch (Exception e) {
				ExceptionDialog.run(e);
			}
	        state.logger().info(L10N.translate("file saved successfully"));
			return;
		}
		
		DocumentFactory documentFactory=state.getDocumentManager().getDocumentFactory(state.getDocumentManager().getMainDocument().getTypeName());
		if(documentFactory==null) {
			state.logger().warning(L10N.translate("internal error: no DocumentFactory found for current main document"));
			return;
		}
    	FileType fileType=documentFactory.getFileType();
    	String filterName=fileType.getName() + " (";
    	for(String pattern : fileType.getPatterns()) {
    		filterName+=pattern + " ";
    	}
    	filterName+=")";
        QFileDialog.Filter filter=new QFileDialog.Filter(filterName);
        String filename=QFileDialog.getSaveFileName(state.getMainWindow(),L10N.translate("Save file"),"",filter);
        
        if(filename.equals("")) { // # if no filename, the operation was canceled
            return;
        }

        state.logger().info(MessageFormat.format(L10N.translate("writing to file {0}"), filename));

        // save document using the document manager
        try {
			state.getDocumentManager().saveMainDocument(filename);
		} catch (Exception e) {
			ExceptionDialog.run(e);
			return;
		}
        state.logger().info(L10N.translate("file saved successfully"));
	}
}
