/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Nov 3, 2005
 */
package eu.moonlight3d.common.ui.fileactions;


import java.text.MessageFormat;

import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;
import eu.moonlight3d.framework.ui.ExceptionDialog;

/**
 * Implementation of the "Save as" action querying the user for
 * a file name under which to save the current main document and
 * saving the document to that file.
 *
 * @author gregor
 */
public class SaveAsAction extends Action {
	/**
	 * Constructor taking a QWidget as a parent
	 * 
	 * @param parent parent widget
	 */
	public SaveAsAction(QWidget parent)
	{
		super("SaveAsAction",L10N.translate("Save &as..."),parent);
		setToolTip(L10N.translate("Save current document to file"));
		setStatusTip(L10N.translate("Save current document to file"));
		triggered.connect(this,"run()");
	}

	@Override
	public void run() {
		State state=State.getInstance();
		
		if(state.getDocumentManager().getMainDocument()==null) {
			return;
		}
		
    	FileType fileType=state.getDocumentManager().getDocumentFactory(state.getDocumentManager().getMainDocument().getTypeName()).getFileType();
    	String filterName=fileType.getName() + " (";
    	for(String pattern : fileType.getPatterns()) {
    		filterName+=pattern + " ";
    	}
    	filterName+=")";
        QFileDialog.Filter filter=new QFileDialog.Filter(filterName);
        String filename=QFileDialog.getSaveFileName(state.getMainWindow(),L10N.translate("Save file"),"",filter);
        
        if(filename.equals("")) { // # if no filename, the operation was canceled
            return;
        }

        state.logger().info(MessageFormat.format(L10N.translate("writing to file {0}"), filename));

        // save document using the document manager
        try {
			state.getDocumentManager().saveMainDocument(filename);
		} catch (Exception e) {
			ExceptionDialog.run(e);
			return;
		}
        state.logger().info(L10N.translate("file saved successfully"));
	}
}
