/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 2, 2006
 */
package eu.moonlight3d.common.ui.preferences;


import com.trolltech.qt.gui.QDialog;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QListWidget;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.ui.propertysheet.PropertySheet;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.preferences.Preferences;
import eu.moonlight3d.framework.preferences.PreferencesGroup;

public class PreferencesDialog extends QDialog {
	private Preferences preferences;
	private PropertySheet preferencesSheet;
	private QListWidget groupTable;

	protected PreferencesDialog(QWidget parent) {
		super(parent);
		preferences=State.getInstance().getPreferencesManager().getCurrentPreferences().clone();
		createContents();
	}

	public void setPreferencesGroup(String groupName) {
		PreferencesGroup preferencesGroup=preferences.getGroupByName(groupName);
		preferencesSheet.setPropertyContainer(preferencesGroup);		
	}

	public void createContents() {
		setWindowTitle(L10N.translate("Preferences"));
		setMinimumSize(300,400);

		groupTable=new QListWidget(this);
		for(PreferencesGroup group : preferences.getGroups()) {
			groupTable.addItem(group.getName());
		}
		groupTable.currentItemChanged.connect(this,"groupItemChanged()");

		preferencesSheet=new PropertySheet(this);
		
		QPushButton cancel=new QPushButton(this);
		cancel.setText(L10N.translate("Cancel"));
		cancel.clicked.connect(this,"handleCancel()");

		QPushButton ok=new QPushButton(this);
		ok.setText(L10N.translate("OK"));
		ok.clicked.connect(this,"handleOK()");
		
		QVBoxLayout layout=new QVBoxLayout();
		QHBoxLayout topLayout=new QHBoxLayout();
		topLayout.addWidget(groupTable,1);
		topLayout.addWidget(preferencesSheet,2);
		QHBoxLayout bottomLayout=new QHBoxLayout();
		bottomLayout.addStretch(1);
		bottomLayout.addWidget(cancel);
		bottomLayout.addWidget(ok);
		
		layout.addLayout(topLayout,1);
		layout.addLayout(bottomLayout);
		
		setLayout(layout);
	}
	
	public void groupItemChanged() {
		String itemName=groupTable.currentItem().text();
		setPreferencesGroup(itemName);		
	}
	
	public void handleOK() {
		State.getInstance().getPreferencesManager().getCurrentPreferences().setFrom(preferences);
		close();		
	}
	
	public void handleCancel() {
		close();
	}
}
