/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 */
package eu.moonlight3d.common.ui.propertysheet;

import java.util.ArrayList;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.trolltech.qt.gui.QCheckBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;

public class BooleanEditorFactory implements PropertyEditorFactory {
	private class Parameters {
		public String labelText;
	}
	
    public class BooleanValueEditor extends ValueEditor {
    	PropertySheet propertySheet;
        Property property;
        QCheckBox checkBox;
        
        BooleanValueEditor(Parameters parameters, PropertySheet sheet, QWidget parent, final Property property)
        {
            super(parent);
            propertySheet=sheet;
            this.property=property;
            
            checkBox=new QCheckBox(this);
            if(parameters.labelText!=null) {
            	checkBox.setText(parameters.labelText);
            } else {
            	checkBox.setText(property.getName());
            }
            checkBox.stateChanged.connect(this,"stateChanged()");
            
            QHBoxLayout layout=new QHBoxLayout();
            layout.setMargin(0);
            layout.addWidget(checkBox);
            setLayout(layout);
            
            updateFromPropertyValue();
        }
        
        public void stateChanged() {
        	setPropertyValue(checkBox.isChecked());
        }
    
        public void setPropertyValue(boolean value)
        {
        	try {
				property.setValue(value);
			} catch (Exception e) {
				// this cannot happen under normal circumstances
				e.printStackTrace();
			}
        	propertySheet.notifyPropertyUpdate(property);
        }

		@Override
		public void updateFromPropertyValue() {
			try {
				checkBox.setChecked(property.getBooleanValue());
			} catch (Exception e) {
				// this cannot happen under normal circumstances.
				e.printStackTrace();
			}
		}
    }

	public String getName() {
		return "BooleanEditor";
	}

	public Object parseExtraParameters(Element parameterRoot) throws Exception {
		Parameters parameters=new Parameters();
		
		if(parameterRoot==null) {
			return parameters;
		}

		NodeList labelList=parameterRoot.getElementsByTagName("label");
		if(labelList.getLength()>1) {
			throw new Exception(L10N.translate("only one label can be defined for a boolean property editor"));
		} else if(labelList.getLength()==1) {
			Element labelElement=(Element) labelList.item(0);
			parameters.labelText=labelElement.getAttribute("text");
		}
		
		return parameters;
	}

	public ValueEditor generatePropertyEditor(PropertySheet sheet, QWidget parent, PropertyContainer container, ArrayList<String> properties, Object additionalParams) {
		Property property=container.getProperty(properties.get(0));
		if(additionalParams==null) {
			additionalParams=new Parameters();
		}
        return new BooleanValueEditor((Parameters) additionalParams,sheet,parent,property);
	}
}
