/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Oct 19, 2005
 */
package eu.moonlight3d.common.ui.propertysheet;

import java.util.ArrayList;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QPaintEvent;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.colour.Colour;
import eu.moonlight3d.common.ui.colourdialog.ColourDialog;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;

public class ColourEditorFactory implements PropertyEditorFactory {
	private class Parameters {
		public String labelText;
		boolean enableAlpha;
	}
	
    public class ColourEditor extends ValueEditor {
    	private boolean enableAlpha;
    	
    	class Canvas extends QWidget {
    		public Canvas(QWidget parent) {
    			super(parent);
    			setMinimumWidth(40);
    		}
    		
    		private QColor getQColor() {
    			Colour colour=null;
    			try {
					colour=property.getColourValue();
				} catch (Exception e) {
					// this cannot happen
					e.printStackTrace();
				}
				Colour.RGB rgb=colour.getsRGB8();
    			return QColor.fromRgb(rgb.red, rgb.green, rgb.blue);    			
    		}
    		
    		@Override 
    		public void paintEvent(QPaintEvent event) {
    			QPainter painter=new QPainter();
    			painter.begin(this);
    			painter.setPen(QColor.black);
    			painter.setBrush(getQColor());
    			painter.drawRect(rect());
    			painter.end();
    		}
    		
    		@Override 
    		public void mouseDoubleClickEvent(QMouseEvent event) {
    			ColourDialog dialog=new ColourDialog(State.getInstance().getUIManager().getActiveApplicationWindow(),property.getColourValue(),enableAlpha);
    			dialog.exec();
    			if(dialog.getOKPressed()) {
    				setPropertyValue(dialog.getColour());
        			update();
    			}
    		}
    	}
    	
    	PropertySheet propertySheet;
        Property property;
        Canvas canvas;

        ColourEditor(Parameters parameters, PropertySheet sheet, QWidget parent, final Property property)
        {
        	super(parent);
        	propertySheet=sheet;
            this.property=property;
            this.enableAlpha=parameters.enableAlpha;
            
            QLabel label=new QLabel(this);
            if(parameters.labelText!=null) {
            	label.setText(parameters.labelText);
            } else {
            	label.setText(property.getName());
            }
            canvas=new Canvas(this);
            
            QHBoxLayout layout=new QHBoxLayout();
            layout.addWidget(label);
            layout.addStretch(1);
            layout.addWidget(canvas);
            layout.setMargin(0);
            setLayout(layout);
        }
        
        public Colour getPropertyValue()
        {
            try {
                return property.getColourValue();            
            } catch(Exception e) {
                // this cannot happen
                return null;
            }
        }
        
        public void setPropertyValue(Colour value)
        {
            try {
                property.setValue(value);
            } catch (Exception e) {
                // this cannot happen
                e.printStackTrace();
            }
            propertySheet.notifyPropertyUpdate(property);
        }

		@Override
		public void updateFromPropertyValue() {
			canvas.repaint();
		}
    }

	public String getName() {
		return "ColourEditor";
	}

	public Object parseExtraParameters(Element parameterRoot) throws Exception {
		Parameters parameters=new Parameters();
		
		if(parameterRoot==null) {
			return parameters;
		}

		NodeList labelList=parameterRoot.getElementsByTagName("label");
		if(labelList.getLength()>1) {
			throw new Exception(L10N.translate("only one label can be defined for a colour property editor"));
		} else if(labelList.getLength()==1) {
			Element labelElement=(Element) labelList.item(0);
			parameters.labelText=labelElement.getAttribute("text");
		}
		
		NodeList alphaList=parameterRoot.getElementsByTagName("alpha");
		if(alphaList.getLength()>1) {
			throw new Exception(L10N.translate("only one alpha tag can be defined for a colour property editor"));
		} else if(alphaList.getLength()==1) {
			Element alphaElement=(Element) alphaList.item(0);
			parameters.enableAlpha=Boolean.parseBoolean(alphaElement.getAttribute("enableAlpha"));
		}
		
		return parameters;
	}

	public ValueEditor generatePropertyEditor(PropertySheet sheet, QWidget parent, PropertyContainer container, ArrayList<String> properties, Object additionalParams) {
		Property property=container.getProperty(properties.get(0));
		if(additionalParams==null) {
			additionalParams=new Parameters();
		}
        return new ColourEditor((Parameters)additionalParams,sheet,parent,property);
	}

}
