/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jul 28, 2006
 */
package eu.moonlight3d.common.ui.propertysheet;

import java.util.ArrayList;

import org.w3c.dom.Element;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QFileDialog;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QListWidget;
import com.trolltech.qt.gui.QListWidgetItem;
import com.trolltech.qt.gui.QPushButton;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.common.ui.propertysheet.IntegerMapFactory.Parameters;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;

public class DirectoryListEditorFactory implements PropertyEditorFactory {

	private class DirectoryListEditor extends ValueEditor {
		private QListWidget directoryTable;
		private Property property;
		private PropertySheet propertySheet;
		

		public DirectoryListEditor(Parameters parameters, PropertySheet sheet, QWidget parent, Property property) {
			super(parent);
			this.property=property;
			this.propertySheet=sheet;
			
			QLabel propertyLabel=new QLabel(this);
			propertyLabel.setText(property.getName());
			
			directoryTable=new QListWidget(this);
			directoryTable.setMinimumHeight(80);
			
			QPushButton add=new QPushButton(this);
			add.setText("Add");
			add.clicked.connect(this,"addEntry()");
			
			QPushButton edit=new QPushButton(this);			
			edit.setText("Edit");
			edit.clicked.connect(this,"editEntry()");

			QPushButton delete=new QPushButton(this);
			delete.setText("Delete");
			delete.clicked.connect(this,"removeSelected()");

			QVBoxLayout layout=new QVBoxLayout();
			QHBoxLayout buttonLayout=new QHBoxLayout();
			buttonLayout.addWidget(add);
			buttonLayout.addStretch(1);
			buttonLayout.addWidget(edit);
			buttonLayout.addStretch(1);
			buttonLayout.addWidget(delete);
			
			layout.addWidget(propertyLabel);
			layout.addWidget(directoryTable,1);
			layout.addLayout(buttonLayout);
			
			setLayout(layout);
			
			updateFromPropertyValue();
		}
		
		public void addEntry() {
			String directory=QFileDialog.getExistingDirectory(eu.moonlight3d.framework.State.getInstance().getMainWindow(),"Select plugin directory");
			
			if(directory==null) {
				return;
			}
			try {
				property.setArrayValue(property.getSize(),directory);
			} catch (Exception e) {
				// cannot happen here
				e.printStackTrace();
			}
			propertySheet.notifyPropertyUpdate(property);
			updateFromPropertyValue();
		}
		
		public void editEntry() {
			if(directoryTable.currentItem()==null) {
				return;
			}
			
			String directory=QFileDialog.getExistingDirectory(eu.moonlight3d.framework.State.getInstance().getMainWindow(),"Select plugin directory",directoryTable.currentItem().text());
			if(directory==null) {
				return;
			}
			if(directory.equals("")) {
				return;
			}

			try {
				property.setArrayValue((Integer)directoryTable.currentItem().data(Qt.ItemDataRole.UserRole),directory);
			} catch (Exception e) {
				// cannot happen here
				e.printStackTrace();
			}
			propertySheet.notifyPropertyUpdate(property);
			updateFromPropertyValue();
		}
		
		public void removeSelected() {
			for(QListWidgetItem item : directoryTable.selectedItems()) {
				int itemIndex=(Integer) item.data(Qt.ItemDataRole.UserRole);
				
				try {
					property.remove(itemIndex);
				} catch (Exception e) {
					// cannot happen here
					e.printStackTrace();
				}
			}
			propertySheet.notifyPropertyUpdate(property);
			updateFromPropertyValue();
		}

		@Override
		public void updateFromPropertyValue() {
			directoryTable.clear();
			for(int i=0;i<property.getSize();i++) {
				try {
					QListWidgetItem item=new QListWidgetItem(directoryTable);
					item.setText(property.getArrayStringValue(i));
					item.setData(Qt.ItemDataRole.UserRole, i);
				} catch (Exception e) {
					// this cannot happen here
					e.printStackTrace();
				}
			}
		}

	}
	
	public String getName() {
		return "DirectoryListEditor";
	}

	public Object parseExtraParameters(Element parameterRoot) throws Exception {
		return null;
	}

	public ValueEditor generatePropertyEditor(PropertySheet sheet, QWidget parent, PropertyContainer container,
			ArrayList<String> properties, Object additionalParams) {
		Property property=container.getProperty(properties.get(0));
		return new DirectoryListEditor((Parameters)additionalParams,sheet,parent,property);
	}

}
