/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 25, 2006
 */
package eu.moonlight3d.common.ui.propertysheet;

import java.util.ArrayList;

import org.w3c.dom.Element;

import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.helper.PropertyContainer;

/**
 * This is a factory for a dummy property editor that does not generate
 * any UI for the property it is assigned. This is useful for hiding
 * properties for which there are specialised and more appropriate 
 * editors elsewhere.
 *
 * @author gregor
 */
public class DummyEditorFactory implements PropertyEditorFactory {

	private class DummyEditor extends ValueEditor {

		public DummyEditor(QWidget parent) {
			super(parent);
			setMinimumSize(0,0);
		}

		@Override
		public void updateFromPropertyValue() {
			// nothing to do here - this is a dummy after all
		}
		
	}
	
	public String getName() {
		// 
		return "DummyEditor";
	}

	public Object parseExtraParameters(Element parameterRoot) throws Exception {
		// a dummy does not have extra parameters
		return null;
	}

	public ValueEditor generatePropertyEditor(PropertySheet sheet, QWidget parent, PropertyContainer container,
			ArrayList<String> properties, Object additionalParams) {
		return new DummyEditor(parent);
	}

}
