/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 24, 2006
 */
package eu.moonlight3d.common.ui.propertysheet;

import com.trolltech.qt.core.QPoint;
import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QApplication;
import com.trolltech.qt.gui.QColor;
import com.trolltech.qt.gui.QFont;
import com.trolltech.qt.gui.QFontMetrics;
import com.trolltech.qt.gui.QFrame;
import com.trolltech.qt.gui.QMouseEvent;
import com.trolltech.qt.gui.QPaintEvent;
import com.trolltech.qt.gui.QPainter;
import com.trolltech.qt.gui.QPolygon;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

public class ExpandItem extends QWidget {
	
	private class Header extends QWidget {
		public Header(QWidget parent) {
			super(parent);
			
			// determine minimal height:
			int textHeight=new QFontMetrics(new QFont()).height();
			setFixedHeight(textHeight+4);
			setMinimumWidth(50);
		}
		
		@Override
		public void mouseReleaseEvent(QMouseEvent event) {
			setExpanded(!expanded);
		}
		
		@Override
		public void paintEvent(QPaintEvent event) {
			super.paintEvent(event);
			
			QPainter painter=new QPainter();
			painter.begin(this);
			// draw gradient frame
//			QLinearGradient gradient=new QLinearGradient(0,0,0,height());
//			if(!expanded) {
//				gradient.setColorAt(0, QColor.darkGray);
//				gradient.setColorAt(1, QColor.darkGray);
//				gradient.setColorAt(2.0/5.0,QColor.lightGray);
//			} else {
//				gradient.setColorAt(0, QColor.lightGray);
//				gradient.setColorAt(1, QColor.lightGray);
//				gradient.setColorAt(3.0/5.0,QColor.darkGray);				
//				gradient.setColorAt(0, QColor.darkGray);
//				gradient.setColorAt(1, QColor.darkGray);
//				gradient.setColorAt(3.0/5.0,QColor.lightGray);				
//			}
//			painter.setBrush(new QBrush(gradient));
//			painter.setPen(Qt.NoPen);
//			painter.drawRect(rect());

			// draw header name
			int textHeight=new QFontMetrics(new QFont()).height();
			painter.setPen(QApplication.palette().text().color());
			painter.drawText(5+textHeight,textHeight,name);
			
			// draw marker triangle
			QPolygon polygon=new QPolygon();
			painter.translate(3+textHeight/2,height()/2);
			polygon.append(new QPoint(0, -height()/2+4));
			polygon.append(new QPoint(0, height()/2-4));
			polygon.append(new QPoint(height()/2-4, 0));
			if(expanded) {
				painter.rotate(90);
			}
			painter.setBrush(QApplication.palette().text().color());
			painter.setPen(Qt.PenStyle.NoPen);
			painter.drawPolygon(polygon);
			painter.end();			
		}
	}
	
	private Header header;
	private QFrame childFrame;
	private QWidget childWidget;
	private String name;
	private boolean expanded;
	private ExpandBar expandBar;
	
	public ExpandItem() {
		super();
//		setFrameStyle(QFrame.Box);

		header=new Header(this);
		childFrame=new QFrame(this);
		// set child frame style
		childFrame.setFrameStyle(QFrame.Shape.NoFrame.value());
		
		expanded=true;
		expandBar=null;
		
		// set layout
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(header);
		layout.addWidget(childFrame);
		layout.setSpacing(0);
		layout.setMargin(0);
		setLayout(layout);
	}

	public void setChildWidget(QWidget childWidget) {
		this.childWidget=childWidget;
		this.childWidget.setParent(childFrame);
		childFrame.setMinimumHeight(childWidget.layout().minimumSize().height());
		QVBoxLayout layout=new QVBoxLayout();
		layout.addWidget(childWidget);
		childFrame.setLayout(layout);
	}
	
	public boolean getExpanded() {
		return expanded;
	}
	
	public void setExpanded(boolean expanded) {
		this.expanded=expanded;
		childFrame.setVisible(expanded);
		if(expanded) {
			childFrame.setMinimumHeight(childFrame.layout().minimumSize().height());
		} else {
			childFrame.setMinimumHeight(0);
		}
		setMinimumHeight(header.minimumHeight()+childFrame.minimumHeight()+2*layout().getContentsMargins().top);
		if(expandBar!=null) {
			expandBar.resetSize();
		}
		header.repaint();
	}
	
	void setExpandBar(ExpandBar expandBar) {
		this.expandBar=expandBar;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
		header.repaint();
	}
}
