/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Sep 25, 2005
 */
package eu.moonlight3d.common.ui.propertysheet;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.trolltech.qt.gui.QDoubleValidator;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.ExceptionDialog;

public class FloatEditorFactory implements PropertyEditorFactory {
	private class Parameters {
		public String labelText;
	}
	
    public class FloatValueEditor extends BasicEditor {
    
        FloatValueEditor(Parameters parameters, PropertySheet propertySheet, QWidget parent, Property property)
        {
            super(propertySheet, parent, property, parameters.labelText);
            
            entry.setValidator(new QDoubleValidator(entry));
        }
        
        @Override
        public void updateFromPropertyValue() {
        	try {
        		double value=NumberFormat.getInstance().parse(entry.text()).doubleValue();
				if(property.getFloatValue()!=value) {
					entry.setText(getPropertyValue());        	
				}
			} catch (NumberFormatException e) {
				entry.setText(getPropertyValue());        	
			} catch (Exception e) {
				entry.setText(getPropertyValue());        	
			}
        }
    
        @Override
        public String getPropertyValue()
        {
            try {
            	DecimalFormat decimalFormat=new DecimalFormat();
            	decimalFormat.applyPattern("0.######");
            	return decimalFormat.format(property.getFloatValue());
            } catch (Exception e) {
                ExceptionDialog.run(e);
                return "";
            }
        }
    
        @Override
        public void setPropertyValue(String value)
        {
            try {
            	double doubleValue=NumberFormat.getInstance().parse(value).doubleValue();
                property.setValue(doubleValue);
            } catch(NumberFormatException e) {
            	// ignore this exception; this may be because the user is still typing 
            } catch(Exception e) {
                ExceptionDialog.run(e);            
            }
        }
    }

	public String getName() {
		return "FloatEditor";
	}

	public Object parseExtraParameters(Element parameterRoot) throws Exception {
		Parameters parameters=new Parameters();
		
		if(parameterRoot==null) {
			return parameters;
		}

		NodeList labelList=parameterRoot.getElementsByTagName("label");
		if(labelList.getLength()>1) {
			throw new Exception(L10N.translate("only one label can be defined for a float property editor"));
		} else if(labelList.getLength()==1) {
			Element labelElement=(Element) labelList.item(0);
			parameters.labelText=labelElement.getAttribute("text");
		}
		
		return parameters;
	}

	public ValueEditor generatePropertyEditor(PropertySheet sheet, QWidget parent, PropertyContainer container, ArrayList<String> properties, Object additionalParams) {
		Property property=container.getProperty(properties.get(0));
		if(additionalParams==null) {
			additionalParams=new Parameters();
		}
        return new FloatValueEditor((Parameters) additionalParams,sheet,parent,property);
	}
}