/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 8, 2006
 */
package eu.moonlight3d.common.ui.propertysheet;

import java.util.ArrayList;
import java.util.HashMap;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.trolltech.qt.gui.QComboBox;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;

public class IntegerMapFactory implements PropertyEditorFactory {

	class Parameters {
		String labelText;
		ArrayList<String> names;
		HashMap<String,Integer> valueMap;
	}

	public class IntegerMapEditor extends ValueEditor {
		PropertySheet propertySheet;
	    Property property;
	    QComboBox entry;
	    Parameters parameters;

	    IntegerMapEditor(Parameters parameters, PropertySheet sheet, QWidget parent, final Property property)
	    {
	    	super(parent);
	    	propertySheet=sheet;
	        this.property=property;
	        this.parameters=parameters;
	        
	        QLabel label;
	        
	        label=new QLabel(this);
	        if(parameters.labelText!=null) {
	        	label.setText(parameters.labelText);
	        } else {
	        	label.setText(property.getName());
	        }

	        entry=new QComboBox(this);
	        // TODO sort this by integer values
	        for(String name : parameters.names) {
	        	entry.addItem(name);
	        }
	        updateFromPropertyValue();
	        entry.setEditable(false);
	        entry.currentIndexChanged.connect(this,"currentIndexChanged()");
	        
	        QHBoxLayout layout=new QHBoxLayout();
	        layout.addWidget(label);
	        layout.addStretch(1);
	        layout.addWidget(entry);
	        layout.setMargin(0);
	        setLayout(layout);
	    }
	    
	    public void currentIndexChanged() {
        	setPropertyValue(entry.itemText(entry.currentIndex()));
            propertySheet.notifyPropertyUpdate(property);	    	
	    }
	    
	    private String getValueName(int value) {
	    	for(String name : parameters.valueMap.keySet()) {
				if(parameters.valueMap.get(name)==value) {
					return name;
				}
	    	}
	    	return "";	    	
	    }
	    
	    @Override
		public void updateFromPropertyValue() {
	    	int itemIndex=0;
	    	int item=0;
	    	try {
				item=property.getIntegerValue();
			} catch (Exception e) {
				// this cannot happen here
				e.printStackTrace();
			}
			String itemName=getValueName(item);
			for(int i=0;i<entry.count();i++) {
				if(entry.itemText(i).equals(itemName)) {
					break;
				}
				itemIndex++;
			}
	   		entry.setCurrentIndex(itemIndex);
	    }
	    
	    public String getPropertyValue() {
	    	try {
				return getValueName(property.getIntegerValue());
			} catch (Exception e) {
				// this cannot happen
				e.printStackTrace();
				return "";
			}
	    }
	    
	    public void setPropertyValue(String value) {
	    	try {
				property.setValue(parameters.valueMap.get(value));
			} catch (Exception e) {
				// this cannot happen here
				e.printStackTrace();
			}
	    }
	}
	
	public String getName() {
		return "IntegerMap";
	}

	public Object parseExtraParameters(Element parameterRoot) throws Exception {
		NodeList valueList=parameterRoot.getElementsByTagName("value");
		Parameters parameters=new Parameters();

		NodeList labelList=parameterRoot.getElementsByTagName("label");
		if(labelList.getLength()>1) {
			throw new Exception(L10N.translate("only one label can be defined for a integer map property editor"));
		} else if(labelList.getLength()==1) {
			Element labelElement=(Element) labelList.item(0);
			parameters.labelText=labelElement.getAttribute("text");
		}

		parameters.names=new ArrayList<String>();
		parameters.valueMap=new HashMap<String,Integer>();
		
		for(int i=0;i<valueList.getLength();i++) {
			Element value=(Element) valueList.item(i);
			parameters.names.add(value.getAttribute("name"));
			parameters.valueMap.put(value.getAttribute("name"),Integer.parseInt(value.getAttribute("value")));
		}
		return parameters;
	}

	public ValueEditor generatePropertyEditor(PropertySheet sheet, QWidget parent, PropertyContainer container,
			ArrayList<String> properties, Object additionalParams) {
		Property property=container.getProperty(properties.get(0));
		if(additionalParams==null) {
			additionalParams=new Parameters();
		}
		return new IntegerMapEditor((Parameters)additionalParams,sheet,parent,property);
	}

}
