/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 7, 2006
 */
package eu.moonlight3d.common.ui.propertysheet;

import java.util.ArrayList;


import org.w3c.dom.Element;

import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.helper.PropertyContainer;

/**
 * Factory interface for property editors. This interface is implemented
 * in factories providing property editors for certain types of properties.
 * The interface is not limited to a one-to-one relationship between properties
 * and property editors, although this is the most common use case.
 *
 * @author gregor
 */
public interface PropertyEditorFactory {
	/**
	 * Return a unique name for this type of property editor.
	 * 
	 * @return the unique, idenitfying name for this type of property editors.
	 */
	public String getName();
	
	/**
	 * Parse parameters for this type of editor from the given part of an XML property 
	 * sheet descrption file and return an object that stores it.
	 * 
	 * @param parameterRoot the root tag for the parameter block in the XML descriptor file 
	 * @return object containing parsed extra parameters
	 * 
	 * @throws Exception
	 */
	public Object parseExtraParameters(Element parameterRoot) throws Exception;
	
	/**
	 * Populate a composite with all the widgets of the property sheet generated from the
	 * given container. The factory also is responsible for installing all relevant listener
	 * so that the sheet can react to user input. The widget listeners are responsible for
	 * calling the notifyPropertyUpdate() method whenever the value of a property has changed.
	 * 
	 * @param sheet the property sheet instance this factory populates
	 * @param parent the parent for all the property sheet widgets
	 * @param container the property container to build the sheet for
	 * @param properties the list of properties for which to build the property editor
	 * @param additionalParams an object containing additional editor parameters as generated from parseExtraParameters()
	 * @return ValueEditor a fully created property editor
	 */
	public ValueEditor generatePropertyEditor(PropertySheet sheet, QWidget parent, PropertyContainer container, ArrayList<String> properties, Object additionalParams);
}
