/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Jan 6, 2008
 */

package eu.moonlight3d.common.ui.propertysheet;

import java.util.HashMap;

import eu.moonlight3d.framework.helper.Value;

/**
 * A set of property editor factories. This stores sets of property
 * editor factories for different types of contexts. Depending on the
 * context in which the property sheet is used, different editors might
 * be desirable and can be passed to a {@link PropertySheet}.  
 *
 * @author gregor
 */
public class PropertyEditorFactorySet {
    /**
     * Return a {@link PropertyEditorFactorySet} which is populated with the set
     * of default editor factories.
     * 
     * @return a {@link PropertyEditorFactorySet} with default editor factories
     */
    public static PropertyEditorFactorySet getDefaultPropertyEditorFactorySet() {
    	PropertyEditorFactorySet defaultSet=new PropertyEditorFactorySet();
    	
        defaultSet.addDefaultPropertyEditorFactory(Value.Type.Boolean,new BooleanEditorFactory());
        defaultSet.addDefaultPropertyEditorFactory(Value.Type.Colour,new ColourEditorFactory());
        defaultSet.addDefaultPropertyEditorFactory(Value.Type.Float,new FloatEditorFactory());
        defaultSet.addDefaultPropertyEditorFactory(Value.Type.Integer,new IntegerEditorFactory());
        defaultSet.addDefaultPropertyEditorFactory(Value.Type.String,new StringEditorFactory());
        defaultSet.addDefaultPropertyEditorFactory(Value.Type.Vector3D,new VectorEditorFactory());

    	return defaultSet;
    }

    /**
     * The default property editors for each value type in this set
     */
    private HashMap<Value.Type,PropertyEditorFactory> defaultPropertyEditorFactories;
    
    /**
     * Default constructor.
     */
    public PropertyEditorFactorySet() {
    	defaultPropertyEditorFactories=new HashMap<Value.Type, PropertyEditorFactory>();
    }
    
    /**
     * Set the default property editor factory for a value type. 
     * 
     * @param type the value type to set the factory for
     * @param editorFactory the factory to use for this value type
     */
    public void addDefaultPropertyEditorFactory(Value.Type type, PropertyEditorFactory editorFactory) {
        defaultPropertyEditorFactories.put(type,editorFactory);    	
    }
    
    /**
     * Get the default property editor factory for a certain type.
     * 
     * @param type the type to get the property editor factory for.
     * @return the property editor factory for this type or null if it is not set
     */
    public PropertyEditorFactory getDefaultPropertyEditorFactory(Value.Type type) {
    	return defaultPropertyEditorFactories.get(type);
    }
    
}
