/*
 * Moonlight|3D Copyright (C) 2006 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on May 7, 2006
 */
package eu.moonlight3d.common.ui.propertysheet;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import eu.moonlight3d.framework.State;

public class PropertySheetDefinition {
	private static ArrayList<PropertySheetDefinition> propertySheetDefinitions;
	
	static {
		propertySheetDefinitions=new ArrayList<PropertySheetDefinition>();		
	}
	
	public static void parsePropertySheetDefinitions() {		
		File definitionsDirectory=new File("propsheet");
		
		File[] definitions = definitionsDirectory.listFiles();
		
		if(definitions==null) {
			return;
		}
		
		for(File file : definitions) {
			if(file.isDirectory()) {
				continue;
			}
			try {
				propertySheetDefinitions.add(new PropertySheetDefinition(file.toURI().toString()));
			} catch (Exception e) {
				// catch errors here to make sure that the offending definition is simply skipped over
				State.getInstance().logger().warning("parse error while reading property sheet definition file " + file.getAbsolutePath());
				e.printStackTrace();
			}
		}
	}
	
	public static ArrayList<PropertySheetDefinition> getPropertySheetDefinitions() {
		return (ArrayList<PropertySheetDefinition>) propertySheetDefinitions.clone();
	}
	
	class EditorDefinition {
		public ArrayList<String> properties;
		public String editor;
		public Object additionalParameters;
	};
	
	class EditorGroup {
		public String caption;
		public ArrayList<EditorDefinition> editors;
	};
	
	private ArrayList<EditorGroup> editorGroups;
	private String typeName;
	private String instanceName;
	
	public PropertySheetDefinition(String filename) throws Exception {
		editorGroups=new ArrayList<EditorGroup>();
		parsePropertySheetDefinition(filename);
	}
	
	public PropertySheetDefinition() {
		editorGroups=new ArrayList<EditorGroup>();		
	}
	
	public void addEditorDefinition(String groupName, EditorDefinition editor) {
		EditorGroup editorGroup=null;
		for(EditorGroup group : editorGroups) {
			if(group.caption.equals(groupName)) {
				editorGroup=group; 
				break;
			}
		}
		
		if(editorGroup==null) {
			editorGroup=new EditorGroup();
			editorGroup.caption=groupName;
			editorGroup.editors=new ArrayList<EditorDefinition>();
			editorGroups.add(editorGroup);
		}
		
		editorGroup.editors.add(editor);
	}
	
	public ArrayList<EditorGroup> getEditorGroups() {
		return (ArrayList<EditorGroup>) editorGroups.clone();
	}
	
	private void parsePropertySheetDefinition(String filename) throws Exception {
        try {
			Document definitionDoc=DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(filename);
			Element rootElement=definitionDoc.getDocumentElement();
			
			typeName=rootElement.getAttribute("typename");
			instanceName=rootElement.getAttribute("instancename");
			
			NodeList editorsList=rootElement.getElementsByTagName("editor");
			
			for(int i=0;i<editorsList.getLength();i++) {
				Element editorElement=(Element) editorsList.item(i);
				String groupName=editorElement.getAttribute("group");
				
				EditorDefinition definition=new EditorDefinition();
				definition.properties=new ArrayList<String>();
				
				NodeList propertyNames=editorElement.getElementsByTagName("property");
				for(int j=0;j<propertyNames.getLength();j++) {
					Element propertyElement=(Element) propertyNames.item(j);
					definition.properties.add(propertyElement.getAttribute("name"));
				}
				
				definition.editor=editorElement.getAttribute("type");
				Element parameterRoot=(Element) editorElement.getElementsByTagName("parameters").item(0);
				for(PropertyEditorFactory factory : PropertySheet.getPropertyEditorFactories()) {
					if(factory.getName().equals(definition.editor)) {
						definition.additionalParameters=factory.parseExtraParameters(parameterRoot);
					}
				}
				
				addEditorDefinition(groupName,definition);
			}
		} catch (SAXException e) {
			throw new Exception("parse error while reading porperty sheet definition",e);
		} catch (IOException e) {
			throw new Exception("parse error while reading porperty sheet definition",e);
		} catch (ParserConfigurationException e) {
			throw new Exception("parse error while reading porperty sheet definition",e);
		}
		
	}
	
	public String getTypeName() {
		return typeName;
	}

	public String getInstanceName() {
		return instanceName;
	}

	public ArrayList<String> knownProperties() {
		ArrayList<String> propertyList=new ArrayList<String>();
		
		for(EditorGroup editorGroup : editorGroups) {
			for(EditorDefinition editor : editorGroup.editors) {
				for(String propertyName : editor.properties) {
					propertyList.add(propertyName);
				}
			}
		}
		
		return propertyList;
	}
}
