/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Sep 26, 2007
 */

package eu.moonlight3d.common.ui.viewactions;

import java.text.MessageFormat;

import com.trolltech.qt.core.QObject;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.Action;
import eu.moonlight3d.framework.ui.ApplicationWindow;


/**
 * Implementation of a the dock widget visibility toggle tools.
 *
 * @author gregor
 */
public class DockWidgetToggleAction extends Action {
	/**
	 * The name of the dock widget for which the
	 * visibility should be toggled.
	 */
	private String dockWidgetName;

	/**
	 * Constructor taking a QWidget as a parent and the
	 * action name
	 * 
	 * @param name action name
	 * @param dockWidgetTitle 
	 * @param parent parent widget
	 */
	public DockWidgetToggleAction(String name, String dockWidgetTitle, QObject parent) {
		super(name + "Action", dockWidgetTitle, parent);
		dockWidgetName=name;
		String toolTipText=MessageFormat.format(L10N.translate("Toggle {0} visibility"), dockWidgetTitle);
		setToolTip(toolTipText);
		setStatusTip(toolTipText);
		setCheckable(true);
		ApplicationWindow currentWindow=State.getInstance().getUIManager().getActiveApplicationWindow();
		setChecked(currentWindow.isVisible(dockWidgetName));
        triggered.connect(this,"run()");
	}
	
	@Override
	public void run() {
		ApplicationWindow currentWindow=State.getInstance().getUIManager().getActiveApplicationWindow();
		currentWindow.toggleDockWidgetVisibility(dockWidgetName);
		setChecked(currentWindow.isVisible(dockWidgetName));
	}

}
