/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Sep 26, 2007
 */

package eu.moonlight3d.common.ui.viewactions;

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.ui.Action;
import eu.moonlight3d.framework.ui.ActionGenerator;
import eu.moonlight3d.framework.ui.FrameLayout;
import eu.moonlight3d.framework.ui.UIManager;

/**
 * Generates toggle dock widget visibility actions for every closeable
 * dock widgets in the {@link FrameLayout} with the given name. 
 *
 * @author gregor
 */
public class DockWidgetToggleActionGenerator extends ActionGenerator {
	/**
	 * {@link FrameLayout} name
	 */
	private String layoutName;

	/**
	 * Constructor taking the name of the {@link FrameLayout} for which
	 * to generate the actions.
	 * 
	 * @param layoutName frame layout name
	 */
	public DockWidgetToggleActionGenerator(String layoutName) {
		super("DockWidgetToggleActionGenerator",ActionGenerator.UpdatePolicy.Always);
		this.layoutName=layoutName;
	}

	@Override
	public ArrayList<Action> getActions() {
		ArrayList<Action> actions=new ArrayList<Action>();
		
		try {
			UIManager manager=State.getInstance().getUIManager();
			FrameLayout layout=manager.getLayout(layoutName);
			
			for(String dockWidgetName : layout.getDockWidgets()) {
				String dockWidgetTitle=manager.getDockWidgetTitle(dockWidgetName);
				actions.add(new DockWidgetToggleAction(dockWidgetName, dockWidgetTitle, null));
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return actions;
	}

}
