/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
package eu.moonlight3d.framework.document;

/**
 * The LifecycleListener is a listener class for document lifecyle
 * events. It is called by the {@link DocumentManager} for
 * lifecycle events of the main document.
 *
 * @author gregor
 */
public interface LifecycleListener {
	/**
	 * Called immediately before a new main document is created.
	 */
	public void beforeNew();
	/**
	 * Called immediately after a new main document is created.
	 */
	public void afterNew();
	/**
	 * Called before the file with the given name is opened.
	 * 
	 * @param filename file name of the file that is opened
	 */
	public void beforeOpen(String filename);
	/**
	 * Called after the file with the given name is opened.
	 * 
	 * @param filename file name of the file that is opened
	 */
	public void afterOpen(String filename);
	/**
	 * Called before the main document is saved to the file
	 * with the given name.
	 * 
	 * @param filename the file name that the document is saved to
	 */
	public void beforeSave(String filename);
	/**
	 * Called after the main document is saved to the file
	 * with the given name.
	 * 
	 * @param filename the file name that the document is saved to
	 */
	public void afterSave(String filename);
	/**
	 * Called before the main document is closed.
	 */
	public void onClose();
}
