/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Nov 2, 2007
 */

package eu.moonlight3d.framework.helper;

import java.util.Arrays;

/**
 * A plain integer array encapsulation designed to reduce the
 * number of array reallocations during the lifetime of the 
 * array.
 * 
 * @author gregor
 */ 
public class BooleanBuffer extends BufferBase implements Cloneable {
	/**
	 * The actual data array.
	 */
	private boolean data[];
	
	/**
	 * Default constructor.
	 */
	public BooleanBuffer() {
		data=new boolean[10];
	}
	
	@Override
	protected void checkCapacity() {
		int newLength=recalculateSize(data.length);
		
		if(newLength>0) {
			data=Arrays.copyOf(data, newLength);
		}
	}
	
	/**
	 * Apend a value to the array.
	 * 
	 * @param data the value to append
	 * @return the index of the value
	 */
	public int append(boolean data) {
		used++;
		if(reserve>0) {
			reserve--;
		}
		// ensure capacity
		checkCapacity();
		
		this.data[used]=data;
		return used-1;
	}
	
	@Override
	public void remove(int elementIndex) {
		for(int i=elementIndex;i<used-1;i++) {
			data[i]=data[i+1];
		}
		used--;
		// check if array could be shrinked
		checkCapacity();		
	}
	
	/**
	 * Get the value of an array element.
	 * 
	 * @param elementIndex the index of the element to return
	 * @return the value of the element
	 */
	public boolean getElement(int elementIndex) {
		return data[elementIndex];
	}
	
	/**
	 * Set the value of an array element.
	 * 
	 * @param elementIndex the index of the element to set
	 * @param newData the new value of the element
	 */
	public void setElement(int elementIndex, boolean newData) {
		data[elementIndex]=newData;
	}
	
	/**
	 * Get the raw buffer buffer data. Use this only if you really
	 * understand how the buffer works internally. Use of
	 * this method is not encouraged because the internals of the
	 * buffer may change in the future because of further 
	 * optimizations that might be possible on it.
	 * 
	 * @return the raw buffer data
	 */
	public boolean[] getRawData() {
		return data;
	}

	/**
	 * Clone this buffer.
	 */
	@Override
	public BooleanBuffer clone() {
		BooleanBuffer cloned;
		try {
			cloned = (BooleanBuffer) super.clone();
			cloned.data=data.clone();
			return cloned;
		} catch (CloneNotSupportedException e) {
			// this cannot happen
			e.printStackTrace();
		}
		return null;
	}
}