/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 5, 2005
 */
package eu.moonlight3d.framework.helper;

import java.util.ArrayList;
import java.util.List;

/**
 * This is a base class for all classes which want to have observers listening
 * to changes.
 * 
 * @author gregor
 */
public class Observed {
	/**
	 * List of registered observers
	 */
    private ArrayList<Observer> observers;
    
    /**
     * Default constructor
     */
    public Observed() 
    {
        observers=new ArrayList<Observer>();
    }

    /**
     * Register an observer
     * 
     * @param observer the observer to register
     */
    public void registerObserver(Observer observer) 
    {
        observers.add(observer);
    }
    
    /**
     * Unregister an observer
     * 
     * @param observer the observer to unregister
     */
    public void unregisterObserver(Observer observer) 
    {
        observers.remove(observer);
    }
       
    /**
     * Return list of all currently registerd observers
     * 
     * @return list of registered observers
     */
    public List<Observer> getObservers() 
    {
        return (ArrayList<Observer>)observers.clone();
    }

    /**
     * Notify all observers.
     */
    public void notifyObservers()
    {
        for(int i=0;i<observers.size();i++) {
            observers.get(i).notify(this);
        }
    }    
}
