/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 5, 2005
 */
package eu.moonlight3d.framework.helper;

import java.util.ArrayList;


/**
 * This is a container class responsible for managing Property classes
 * and making them available by name.
 * 
 * @author gregor
 */
public class PropertyContainer {
    private ArrayList<Property> properties;
    private boolean hasUpdatedProperties;
    
    public PropertyContainer()
    {
        properties=new ArrayList<Property>();
        hasUpdatedProperties=false;
    }

    public void registerProperty(Property p)
    {
        properties.add(p);
    }
    
    public void unregisterProperty(Property p)
    {
        properties.remove(p);
    }
    
    public int getNumProperties() {
    	return properties.size();
    }

    public ArrayList<String> getPropertyList()
    {
        ArrayList<String> propertyNames=new ArrayList<String>();
        
        for(int i=0;i<properties.size();i++) {
            propertyNames.add(properties.get(i).getName());
        }
        
        return propertyNames;
    }
    
    public Property getProperty(String name)
    {
        for(int i=0;i<properties.size();i++) {
            if(properties.get(i).getName().equals(name)) {
                return properties.get(i);
            }
        }
        
        return null;
    }
    
    public void setHasUpdatedProperties() {
    		hasUpdatedProperties=true;
    }
    
    public void clearHasUpdatedProperties() {
    		hasUpdatedProperties=false;
    }

    public boolean hasUpdatedProperties() {
    		return hasUpdatedProperties;
    }
}
