/*
 * Moonlight|3D Copyright (C) 2005 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Dec 29, 2005
 */
package eu.moonlight3d.framework.preferences;


import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import eu.moonlight3d.framework.helper.NamedObject;
import eu.moonlight3d.framework.helper.Property;
import eu.moonlight3d.framework.helper.PropertyContainer;

public class PreferencesGroup extends PropertyContainer implements Cloneable, NamedObject {
	protected String name;
	
	public PreferencesGroup() {
		super();
	}

	public void setName(String name) {
		this.name=name;
	}
	
	public String getName() {
		return name;
	}
	
	public void setFrom(PreferencesGroup preferencesGroup) {
		for(String propertyName : preferencesGroup.getPropertyList()) {
			Property thisProperty=getProperty(propertyName);
			
			// if property doesn't exist in this group, skip it
			if(thisProperty==null) {
				continue;
			}
			
			Property thatProperty=preferencesGroup.getProperty(propertyName);
			
			if(thisProperty.getType()!=thatProperty.getType()) {
				continue;
			}
			
			thisProperty.copyFrom(thatProperty);
		}
	}

	public void readFromFile(Element groupNode) {
		NodeList propertyNodes=groupNode.getElementsByTagName("property");
		
		for(int i=0;i<propertyNodes.getLength();i++) {
			Element propertyNode=(Element) propertyNodes.item(i);
			Property property=new Property(this);
			try {
				property.readFromFile(propertyNode,false);
			} catch (Exception e) {
				// TODO: handle exception properly here:
				e.printStackTrace();
			}
		}
	}

	public void saveToFile(Document preferencesDoc, Element root) throws DOMException, Exception {
		Element groupElement=preferencesDoc.createElement("group");
		groupElement.setAttribute("name",name);
		for(int i=0; i<getPropertyList().size();i++) {
			getProperty(getPropertyList().get(i)).writeToFile(preferencesDoc,groupElement);
		}
		root.appendChild(groupElement);
	}
	
	@Override
	public PreferencesGroup clone() {
		PreferencesGroup group=new PreferencesGroup();
		
		group.name=name;
		for(String propertyName : getPropertyList()) {
			Property property=new Property(group);
			property.setName(propertyName);
			property.setValue(getProperty(propertyName));
		}
		
		return group;
	}
}
