/*
 * Moonlight|3D
 * Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Sep 23, 2007
 */

package eu.moonlight3d.framework.ui;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QDockWidget;
import com.trolltech.qt.gui.QWidget;

/**
 * A QWidget subclass used in the creation of dock widgets for
 * application windows. It keeps additional required information 
 * for the dock widget creation through {@link ApplicationWindow}.
 *
 * @author gregor
 */
public class DockWidget extends QWidget {
	/**
	 * Dock widget title
	 */
	private String title;
	/**
	 * Allowed aras for the dock widget in the application window
	 */
	private Qt.DockWidgetAreas allowedAreas;
	/**
	 * Default placement area for the dock widget
	 */
	private Qt.DockWidgetArea defaultArea;
	/**
	 * Dock widget features which are enabled
	 */
	private QDockWidget.DockWidgetFeatures features;
	
	/**
	 * Default constructor.
	 */
	public DockWidget() {
		title="UNNAMED";
		allowedAreas=new Qt.DockWidgetAreas(Qt.DockWidgetArea.LeftDockWidgetArea,
											Qt.DockWidgetArea.RightDockWidgetArea,
											Qt.DockWidgetArea.TopDockWidgetArea,
											Qt.DockWidgetArea.BottomDockWidgetArea);
		
		defaultArea=Qt.DockWidgetArea.LeftDockWidgetArea;
		features=new QDockWidget.DockWidgetFeatures(QDockWidget.DockWidgetFeature.DockWidgetClosable,
													QDockWidget.DockWidgetFeature.DockWidgetFloatable,
													QDockWidget.DockWidgetFeature.DockWidgetMovable);
	}
	
	/**
	 * Constructor taking the dock widget title as argument. 
	 * 
	 * @param title dock widget title
	 */
	public DockWidget(String title) {
		this.title=title;

		allowedAreas=new Qt.DockWidgetAreas(Qt.DockWidgetArea.LeftDockWidgetArea,
											Qt.DockWidgetArea.RightDockWidgetArea,
											Qt.DockWidgetArea.TopDockWidgetArea,
											Qt.DockWidgetArea.BottomDockWidgetArea);

		defaultArea=Qt.DockWidgetArea.LeftDockWidgetArea;
		features=new QDockWidget.DockWidgetFeatures(QDockWidget.DockWidgetFeature.DockWidgetClosable,
													QDockWidget.DockWidgetFeature.DockWidgetFloatable,
													QDockWidget.DockWidgetFeature.DockWidgetMovable);
	}

	/**
	 * Get dock widget title.
	 * 
	 * @return dock widget title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * Set dock widget title.
	 * 
	 * @param title dock widget title
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * Get allowed areas for this dock widget.
	 * 
	 * @return allowed areas
	 */
	public Qt.DockWidgetAreas getAllowedAreas() {
		return allowedAreas;
	}

	/**
	 * Set allowed areas for this dock widget.
	 * 
	 * @param allowedAreas new allowed areas
	 */
	public void setAllowedAreas(Qt.DockWidgetAreas allowedAreas) {
		this.allowedAreas = allowedAreas;
	}

	/**
	 * Get default placement area.
	 * 
	 * @return default area
	 */
	public Qt.DockWidgetArea getDefaultArea() {
		return defaultArea;
	}

	/**
	 * Set default placement area.
	 * 
	 * @param defaultArea default placement area
	 */
	public void setDefaultArea(Qt.DockWidgetArea defaultArea) {
		this.defaultArea = defaultArea;
	}

	/**
	 * Get enabled features for this dock widget.
	 * 
	 * @return enabled features
	 */
	public QDockWidget.DockWidgetFeatures getFeatures() {
		return features;
	}

	/**
	 * Set enabled features for this dock widget.
	 * 
	 * @param features enabled features
	 */
	public void setFeatures(QDockWidget.DockWidgetFeatures features) {
		this.features = features;
	}
}
