/*
 * Moonlight|3D
 * Copyright (C) 2008 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Feb 5, 2008
 */

package eu.moonlight3d.framework.helper.test;

import java.util.Random;

import eu.moonlight3d.framework.helper.IntArrayBuffer;
import junit.framework.TestCase;

/**
 * Some test cases for  IntArrayBuffer.
 *
 * @author gregor
 */
public class IntArrayBufferTest extends TestCase {
	/**
	 * Tests addition and removal of arrays in several
	 * situations. It assures the correct preservation
	 * of all other arrays in the buffer. 
	 */
	public void testAddAndRemove() {
		IntArrayBuffer buffer=new IntArrayBuffer();
		
		for(int i=0;i<20;i++) {
			int[] array=new int[i];
			for(int j=0;j<i;j++) {
				array[j]=j;
			}
			buffer.append(array);
		}
		
		int[] shockArray=new int[0];
		buffer.append(shockArray);
		
		assertTrue(buffer.getNumArrays()==21);
		assertTrue(buffer.getLength()==190);
		
		// test retrieval
		for(int i=0;i<20;i++) {
			int[] array=buffer.getArray(i);
			assertTrue(array.length==i);
			for(int j=0;j<i;j++) {
				assertTrue(array[j]==j);
			}
		}
		
		// test removal
		buffer.remove(0);
		assertTrue(buffer.getNumArrays()==20);
		assertTrue(buffer.getLength()==190);

		for(int i=1;i<19;i++) {
			int[] array=buffer.getArray(i-1);
			assertTrue(array.length==i);
			for(int j=0;j<i;j++) {
				assertTrue(array[j]==j);
			}
		}
		
		buffer.remove(19);
		assertTrue(buffer.getNumArrays()==19);
		assertTrue(buffer.getLength()==190);

		for(int i=1;i<18;i++) {
			int[] array=buffer.getArray(i-1);
			assertTrue(array.length==i);
			for(int j=0;j<i;j++) {
				assertTrue(array[j]==j);
			}
		}
		
		for(int k=17;k>=10;k--) {
			buffer.remove(k);
			assertTrue(buffer.getNumArrays()==k+1);
			for(int i=1;i<k;i++) {
				int[] array=buffer.getArray(i-1);
				assertTrue(array.length==i);
				for(int j=0;j<i;j++) {
					assertTrue(array[j]==j);
				}
			}
		}

		// test replacement
		for(int i=0;i<10;i++) {
			int[] array=new int[i];
			for(int j=0;j<i;j++) {
				array[j]=j;
			}
			buffer.setArray(i, array);
		}
		for(int i=0;i<10;i++) {
			int[] array=buffer.getArray(i);
			assertTrue(array.length==i);
			for(int j=0;j<i;j++) {
				assertTrue(array[j]==j);
			}
		}
		
		assertTrue(buffer.getArrayLength(10)==19);
		assertTrue(buffer.getArray(10).length==19);
	}
	
	/**
	 * Tests addition and removal of arrays in several
	 * situations. It assures the correct preservation
	 * of all other arrays in the buffer. 
	 */
	public void testAddAndRemove2() {
		IntArrayBuffer buffer=new IntArrayBuffer();

		int[] firstArray=new int[10];
		int[] secondArray=new int[20];
		int[] thirdArray=new int[10];
		
		Random random=new Random();
		for(int i=0;i<10;i++) {
			firstArray[i]=random.nextInt();
			thirdArray[i]=random.nextInt();
		}
		for(int i=0;i<20;i++) {
			secondArray[i]=random.nextInt();
		}
		
		buffer.append(firstArray);
		buffer.append(secondArray);
		buffer.append(thirdArray);
		
		assertTrue(buffer.getNumArrays()==3);
		
		buffer.remove(1);
		assertTrue(buffer.getNumArrays()==2);
		assertTrue(buffer.getArrayLength(0)==10);
		assertTrue(buffer.getArrayLength(1)==10);
		for(int i=0;i<10;i++) {
			assertTrue(firstArray[i]==buffer.getArrayElement(0,i));
			assertTrue(thirdArray[i]==buffer.getArrayElement(1,i));
		}
		
		buffer.remove(1);
		assertTrue(buffer.getNumArrays()==1);
		assertTrue(buffer.getArrayLength(0)==10);
		for(int i=0;i<10;i++) {
			assertTrue(firstArray[i]==buffer.getArrayElement(0,i));
		}
	}

	/**
	 * Tests resize() in several different situations
	 */
	public void testResize() {
		// 1. build array buffer with three arrays
		IntArrayBuffer buffer=new IntArrayBuffer();
		int[] data=new int[2];
		data[0]=1;
		data[1]=2;
		for(int i=0;i<3;i++) {
			buffer.append(data);
		}
		// 2. grow each array
		buffer.resize(0,3);
		buffer.setArrayElement(0, 2, 3);
		
		// check buffer contents
		assertTrue(buffer.getArrayElement(0, 2)==3);		
		for(int i=0;i<3;i++) {
			data=buffer.getArray(i);
			for(int j=0;j<2;j++) {
				assertTrue(data[j]==j+1);
			}
		}
		
		buffer.resize(1,3);
		buffer.setArrayElement(1, 2, 3);

		// check buffer contents
		assertTrue(buffer.getArrayElement(0, 2)==3);		
		assertTrue(buffer.getArrayElement(1, 2)==3);		
		for(int i=0;i<3;i++) {
			data=buffer.getArray(i);
			for(int j=0;j<2;j++) {
				assertTrue(data[j]==j+1);
			}
		}
		
		buffer.resize(2,3);
		buffer.setArrayElement(2, 2, 3);

		// check buffer contents
		for(int i=0;i<3;i++) {
			for(int j=0;j<3;j++) {
				assertTrue(buffer.getArrayElement(i,j)==j+1);
			}
		}
		
		// 3. shrink each array
		buffer.resize(0,2);
		// check buffer contents
		assertTrue(buffer.getArrayElement(1, 2)==3);		
		assertTrue(buffer.getArrayElement(2, 2)==3);		
		for(int i=0;i<3;i++) {
			data=buffer.getArray(i);
			for(int j=0;j<2;j++) {
				assertTrue(data[j]==j+1);
			}
		}
		
		buffer.resize(1,2);		
		// check buffer contents
		assertTrue(buffer.getArrayElement(2, 2)==3);		
		for(int i=0;i<3;i++) {
			data=buffer.getArray(i);
			for(int j=0;j<2;j++) {
				assertTrue(data[j]==j+1);
			}
		}

		buffer.resize(2,2);
		// check buffer contents
		for(int i=0;i<3;i++) {
			data=buffer.getArray(i);
			for(int j=0;j<2;j++) {
				assertTrue(data[j]==j+1);
			}
		}
	}
}
