/*
 * Moonlight|3D
 * Copyright (C) 2009 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Sep 16, 2009
 */

package eu.moonlight3d.hdrviewer.document;

import java.util.ArrayList;

import eu.moonlight3d.common.image.core.FormatManager;
import eu.moonlight3d.common.image.core.ImageFormat;
import eu.moonlight3d.framework.document.Document;
import eu.moonlight3d.framework.document.DocumentFactory;
import eu.moonlight3d.framework.helper.FileType;
import eu.moonlight3d.framework.helper.PropertyContainer;
import eu.moonlight3d.framework.localization.L10N;

public class ImageDocumentFactory implements DocumentFactory {

	public boolean canCreateNew() {
		return false;
	}

	public boolean canSaveFiles() {
		return false;
	}

	public boolean canOpenDocument(String filename) {
		FormatManager formatManager=FormatManager.getInstance();
		for(ImageFormat format : formatManager.getFormats()) {
			if(format.isLoadable()) {
				try {
					if(format.canLoadFile(filename)) {
						return true;
					}
				} catch (Exception e) {
					continue; // ignore exception and try next format
				}
			}
		}
		return false;
	}

	public Document createNewDocument() {
		// TODO Auto-generated method stub
		return null;
	}

	public Document createNewDocument(PropertyContainer options) {
		// TODO Auto-generated method stub
		return null;
	}

	public PropertyContainer createNewDocumentProperties() {
		return null;
	}

	public String getDisplayName() {
		return L10N.translate("High Dynamic Range Image");
	}

	public FileType getFileType() {
		ArrayList<String> patterns=new ArrayList<String>();
		
		// insert patterns from image formats
		FormatManager formatManager=FormatManager.getInstance();
		for(ImageFormat format : formatManager.getFormats()) {
			if(format.isLoadable()) {
				patterns.addAll(format.getFileType().getPatterns());
			}
		}
		FileType fileType=new FileType(L10N.translate("High Dynamic Range Image"),patterns);
		return fileType;
	}

	public String getTypeName() {
		return "HDRImage";
	}

	public Document openDocument(String filename) throws Exception {
		return new ImageDocument(filename);
	}

}
