/*
 * Moonlight|3D Copyright (C) 2007 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Created on Jan 25, 2007
 */
package eu.moonlight3d.hdrviewer.mainwindow;

import com.trolltech.qt.core.Qt;
import com.trolltech.qt.gui.QHBoxLayout;
import com.trolltech.qt.gui.QLabel;
import com.trolltech.qt.gui.QSlider;
import com.trolltech.qt.gui.QVBoxLayout;
import com.trolltech.qt.gui.QWidget;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.hdrviewer.document.ImageDocument;
import eu.moonlight3d.hdrviewer.viewer.ViewWidget;

/**
 * The main window's main widget. It contains the the tab widget
 * with the 3d views and the operator graph editor
 *
 * @author gregor
 */
public class MainWindow extends QWidget {
	private QSlider exposureSlider;
	private ViewWidget viewWidget;
	
	/**
	 * Default constructor
	 * 
	 * @param parent the parent widget
	 */
	MainWindow(QWidget parent) {
		super(parent);
		
		QLabel exposureLabel=new QLabel(this);
		exposureLabel.setText(L10N.translate("Exposure"));
		exposureSlider=new QSlider(this);
		exposureSlider.setRange(1,500);
		exposureSlider.setValue(100);
		exposureSlider.setOrientation(Qt.Orientation.Horizontal);
		exposureSlider.valueChanged.connect(this,"exposureChanged()");
		
		QHBoxLayout topLayout=new QHBoxLayout();
		topLayout.addWidget(exposureLabel);
		topLayout.setStretchFactor(exposureLabel, 0);
		topLayout.addWidget(exposureSlider);
		topLayout.setStretchFactor(exposureSlider, 1);
		
		viewWidget=new ViewWidget(this);
		
		QVBoxLayout layout=new QVBoxLayout(this);
		layout.addLayout(topLayout);
		layout.setStretchFactor(topLayout, 0);
		layout.addWidget(viewWidget);
		layout.setStretchFactor(viewWidget,1);
		layout.setMargin(0);
		setLayout(layout);
		
		LifecycleListener.registerMainWindow(this);
		
		// check if there was already an image opened during startup and display it
		State state=State.getInstance();
		if(state.getDocumentManager().getMainDocument()!=null) {
			ImageDocument document=(ImageDocument) state.getDocumentManager().getMainDocument();
			if(document.getImage()!=null) {
				viewWidget.setImage(document.getImage());
			}
		}
	}
	
	protected void exposureChanged() {
		float exposure=exposureSlider.value()/100.0f;
		viewWidget.setExposure(exposure);
	}

	/**
	 * Get the view area widget
	 * 
	 * @return the view area widget
	 */
	public ViewWidget getView() {
		return viewWidget;
	}
	
	@Override
	public void disposed() {		
		LifecycleListener.unregisterMainWindow(this);
	}
}
