/*
 * Moonlight|3D
 * Copyright (C) 2009 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Sep 16, 2009
 */

package eu.moonlight3d.hdrviewer.mainwindow;

import java.util.ArrayList;

import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;
import eu.moonlight3d.framework.ui.AboutDialog;
import eu.moonlight3d.framework.ui.MenuNode;
import eu.moonlight3d.framework.ui.UIManager;

public class Plugin implements eu.moonlight3d.framework.plugins.Plugin {

	private LifecycleListener lifecycleListener;
	private HDRViewerAboutDialogPage aboutDialogPage;

	public ArrayList<String> getDependencies() {
		ArrayList<String> dependencyList=new ArrayList<String>();
		dependencyList.add("eu.moonlight3d.hdrviewer.document.Plugin");
		dependencyList.add("eu.moonlight3d.hdrviewer.viewer.Plugin");
		return dependencyList;
	}

	public String getName() {
		return "eu.moonlight3d.hdrviewer.mainwindow.Plugin";
	}

	public void load() throws Exception {
		State state=State.getInstance();
		
		if(!state.getBatchMode()) {
			lifecycleListener=new LifecycleListener();
			state.getDocumentManager().addLifecycleListener(lifecycleListener);
			
			aboutDialogPage=new HDRViewerAboutDialogPage();
			AboutDialog.setMainAboutDialogPageFactory(aboutDialogPage);

        	UIManager uiManager=state.getUIManager();
			uiManager.registerMainWidgetFactory(new MainWindowFactory());
			
			uiManager.registerAction(new ZoomInAction(null));
			uiManager.registerAction(new ZoomOutAction(null));
			uiManager.registerAction(new ZoomLevelAction(null,"ZoomLevel25Action",0.25f));
			uiManager.registerAction(new ZoomLevelAction(null,"ZoomLevel50Action",0.5f));
			uiManager.registerAction(new ZoomLevelAction(null,"ZoomLevel75Action",0.75f));
			uiManager.registerAction(new ZoomLevelAction(null,"ZoomLevel100Action",1.0f));
			uiManager.registerAction(new ZoomLevelAction(null,"ZoomLevel150Action",1.5f));
			uiManager.registerAction(new ZoomLevelAction(null,"ZoomLevel200Action",2.0f));
			uiManager.registerAction(new ZoomLevelAction(null,"ZoomLevel300Action",3.0f));
			uiManager.registerAction(new ZoomLevelAction(null,"ZoomLevel400Action",4.0f));
			
			MenuNode contribution=new MenuNode(MenuNode.Type.Contribution,"View","menubar");
			MenuNode viewMenu=new MenuNode(MenuNode.Type.Menu,L10N.translate("View"),"");
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom in"),"ZoomInAction"));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom out"),"ZoomOutAction"));
			viewMenu.add(new MenuNode(MenuNode.Type.Separator,"",""));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom 25%"),"ZoomLevel25Action"));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom 50%"),"ZoomLevel50Action"));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom 75%"),"ZoomLevel75Action"));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom 100%"),"ZoomLevel100Action"));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom 150%"),"ZoomLevel150Action"));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom 200%"),"ZoomLevel200Action"));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom 300%"),"ZoomLevel300Action"));
			viewMenu.add(new MenuNode(MenuNode.Type.Entry,L10N.translate("Zoom 400%"),"ZoomLevel400Action"));
			contribution.add(viewMenu);
			uiManager.getLayout("mainwindow").addMenuContribution(contribution);
		}
	}

	public void unload() throws Exception {
		State state=State.getInstance();
		
		if(!state.getBatchMode()) {
			state.getDocumentManager().removeLifecycleListener(lifecycleListener);			
			
			AboutDialog.setMainAboutDialogPageFactory(null);

			UIManager uiManager=state.getUIManager();
        	uiManager.unregisterMainWidgetFactory("mainwindow");
		}
	}

}
