/*
 * Moonlight|3D
 * Copyright (C) 2009 The Moonlight|3D team
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Created on Sep 16, 2009
 */

package eu.moonlight3d.hdrviewer.viewer;

import java.io.IOException;
import java.io.InputStream;
import java.nio.FloatBuffer;

import javax.media.opengl.GL;
import javax.media.opengl.GLContext;
import javax.media.opengl.GLDrawableFactory;
import javax.media.opengl.glu.GLU;

import com.trolltech.qt.gui.QWidget;
import com.trolltech.qt.opengl.QGLFormat;
import com.trolltech.qt.opengl.QGLWidget;

import eu.moonlight3d.common.image.core.Channel;
import eu.moonlight3d.common.image.core.Framebuffer;
import eu.moonlight3d.framework.State;
import eu.moonlight3d.framework.localization.L10N;

public class HDRDisplay extends QGLWidget {
	private GLContext glContext;
	private GL gl;
	private GLU glu;
	
	private String displayFragmentSource;
	private String displayVertexSource;
	private int displayProgramID;
	private int displayFragmentID;
	private int displayVertexID;
	
	private int imageTexture;
	private Framebuffer image;
	
	private float exposure;
	private float gamma;
	
	private double scale;

	public HDRDisplay(QWidget parent, QGLFormat format) {
		super(format, parent);
		
		exposure=1.0f;
		gamma=2.2f;
		scale=1.0f;
	}
	
	private String loadShader(String filename) {		
		try {
			InputStream input=this.getClass().getResourceAsStream(filename);
			byte[] buffer=new byte[(int) input.available()];
			input.read(buffer);
			
			return new String(buffer);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	private void updateTexture() {
		if(image==null) {
			return;
		}
		
		// gl may not be initialised yet when update is requested
		if(gl==null) {
			return;
		}

		// TODO handle case of monochrome images
		
		// create GL_FLOAT_RGBA texture from image
        Channel redChannel=image.getChannel("red");
		Channel greenChannel=image.getChannel("green");
		Channel blueChannel=image.getChannel("blue");
		if(image.getChannel("alpha")!=null) {
			Channel alphaChannel=image.getChannel("alpha");
			float[] openGLTexture=new float[4*image.getWidth()*image.getHeight()];
			int index=0;
			for(int y=0;y<image.getHeight();y++) {
				for(int x=0;x<image.getWidth();x++) {
					openGLTexture[index]=redChannel.getPixel(x,y);
					index++;
					openGLTexture[index]=greenChannel.getPixel(x,y);
					index++;
					openGLTexture[index]=blueChannel.getPixel(x,y);
					index++;
					openGLTexture[index]=alphaChannel.getPixel(x, y);
					index++;
				}
			}
			FloatBuffer buffer=FloatBuffer.allocate(openGLTexture.length);
			buffer.put(openGLTexture);
			buffer.rewind();
			gl.glTexImage2D(GL.GL_TEXTURE_2D,0,GL.GL_RGBA32F_ARB,image.getWidth(),image.getHeight(),0,GL.GL_RGBA,GL.GL_FLOAT,buffer);
		} else {
			float[] openGLTexture=new float[3*image.getWidth()*image.getHeight()];
			int index=0;
			for(int y=0;y<image.getHeight();y++) {
				for(int x=0;x<image.getWidth();x++) {
					openGLTexture[index]=redChannel.getPixel(x,y);
					index++;
					openGLTexture[index]=greenChannel.getPixel(x,y);
					index++;
					openGLTexture[index]=blueChannel.getPixel(x,y);
					index++;
				}
			}				
			FloatBuffer buffer=FloatBuffer.allocate(openGLTexture.length);
			buffer.put(openGLTexture);
			buffer.rewind();
			gl.glTexImage2D(GL.GL_TEXTURE_2D,0,GL.GL_RGB32F_ARB,image.getWidth(),image.getHeight(),0,GL.GL_RGB,GL.GL_FLOAT,buffer);
		}
		
		State.getInstance().logger().info("image size: " + image.getWidth() + "x" + image.getHeight());
		resize((int)(scale*image.getWidth()),(int)(scale*image.getHeight()));
	}
	
	@Override
	protected void initializeGL() {
        if(glContext==null) {
        	glContext=GLDrawableFactory.getFactory().createExternalGLContext();
        }
        makeCurrent();
        glContext.makeCurrent();
        gl=glContext.getGL();
        glu=new GLU();
        
        int temp[]=new int[1];
        gl.glGenTextures(1, temp, 0);
        imageTexture=temp[0];
//        gl.glActiveTexture(imageTexture);
        
		gl.glBindTexture(GL.GL_TEXTURE_2D,imageTexture);
		
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_S, GL.GL_CLAMP);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_WRAP_T, GL.GL_CLAMP);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_NEAREST);
        gl.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_NEAREST);
        
        
        // load and compile shader
        displayFragmentSource=loadShader("display_fragment.glsl");
        displayVertexSource=loadShader("display_vertex.glsl");
        
		displayProgramID = gl.glCreateProgram();
		displayVertexID = gl.glCreateShader(GL.GL_VERTEX_SHADER);
		displayFragmentID = gl.glCreateShader(GL.GL_FRAGMENT_SHADER);
		
		String[] shaderBuffer = new String[1];

		shaderBuffer[0] = displayVertexSource;
		gl.glShaderSource(displayVertexID, 1, shaderBuffer, null);
		shaderBuffer[0] = displayFragmentSource;
		gl.glShaderSource(displayFragmentID, 1, shaderBuffer, null);

		byte[] errorBuffer = new byte[8192];
		int[] messageLength = new int[1];

		gl.glCompileShader(displayVertexID);
		gl.glGetShaderInfoLog(displayVertexID, 8192, messageLength, 0, errorBuffer, 0);
		if (messageLength[0] > 0) {
			eu.moonlight3d.framework.State.getInstance().logger().warning(
					L10N.translate("GLSL vertex program compiler messages: ") + new String(errorBuffer, 9, messageLength[0]));
		}
		gl.glCompileShader(displayFragmentID);
		gl.glGetShaderInfoLog(displayFragmentID, 8192, messageLength, 0, errorBuffer, 0);
		if (messageLength[0] > 0) {
			eu.moonlight3d.framework.State.getInstance().logger().warning(
					L10N.translate("GLSL fragment program compiler messages: ") + new String(errorBuffer, 0, messageLength[0]));
		}

		gl.glAttachShader(displayProgramID, displayVertexID);
		gl.glAttachShader(displayProgramID, displayFragmentID);
		gl.glLinkProgram(displayProgramID);
		gl.glGetProgramInfoLog(displayProgramID, 8192, messageLength, 0, errorBuffer, 0);
		if (messageLength[0] > 0) {
			eu.moonlight3d.framework.State.getInstance().logger().warning(
					L10N.translate("GLSL program linker messages: ") + new String(errorBuffer, 0, messageLength[0]));
		}
		
		updateTexture();
	}
	
	public void setImage(Framebuffer image) {
		this.image=image;
		updateTexture();
		updateGL();
	}
	
	public void setGamma(float gamma) {
		this.gamma=gamma;
		updateGL();
	}
	
	public float getGamma() {
		return gamma;
	}
	
	public void setExposure(float exposure) {
		this.exposure=exposure;
		updateGL();
	}
	
	public float getExposure() {
		return exposure;
	}
	
	public void setScale(double scale) {
		this.scale=scale;
		resize((int)(scale*image.getWidth()),(int)(scale*image.getHeight()));
		updateGL();		
	}
	
	public double getScale() {
		return scale;
	}

	@Override
	protected void paintGL() {
		gl.glViewport(0, 0, width(), height());
		gl.glMatrixMode(GL.GL_MODELVIEW);
		gl.glLoadIdentity();
		gl.glMatrixMode(GL.GL_PROJECTION);
		gl.glLoadIdentity();
		gl.glOrtho(0, width(), height(), 0, -100.0, 100.0);
		
		gl.glUseProgram(displayProgramID);
		
		int uniformIndex=gl.glGetUniformLocation(displayProgramID,"exposure");
		if(uniformIndex==-1) {
			String message=L10N.translate("uniform shader parameter exposure does not exist");
			State.getInstance().logger().warning(message);
		}
		gl.glUniform1f(uniformIndex,(float)exposure);						

		uniformIndex=gl.glGetUniformLocation(displayProgramID,"gamma");
		if(uniformIndex==-1) {
			String message=L10N.translate("uniform shader parameter gamma does not exist");
			State.getInstance().logger().warning(message);
		}
		gl.glUniform1f(uniformIndex,(float)gamma);
		
		uniformIndex=gl.glGetUniformLocation(displayProgramID,"image");
		if(uniformIndex==-1) {
			String message=L10N.translate("uniform shader parameter image does not exist");
			State.getInstance().logger().warning(message);
		}
		gl.glActiveTexture(GL.GL_TEXTURE0);
		gl.glBindTexture(GL.GL_TEXTURE_2D, imageTexture);
		gl.glUniform1i(uniformIndex,0);
		
//		gl.glUseProgram(0);
//		gl.glColor3d(1.0,0.0,0.0);
		
		gl.glBegin(GL.GL_QUADS);
		gl.glTexCoord2d(0.0, 0.0);
		gl.glVertex3d(0, 0, 0);
		gl.glTexCoord2d(0.0, 1.0);
		gl.glVertex3d(0, height(), 0);
		gl.glTexCoord2d(1.0, 1.0);
		gl.glVertex3d(width(), height(), 0);
		gl.glTexCoord2d(1.0, 0.0);
		gl.glVertex3d(width(), 0, 0);
		gl.glEnd();
		
		gl.glUseProgram(0);
	}
}
